#!/bin/bash

are_dl_processes_running()
{
    pgrep DisplayLink &> /dev/null
}

terminate_any_remaining_dl_processes()
{
  if are_dl_processes_running; then
    echo "Terminate remaining DisplayLink processes"
    killall -q -SIGTERM 'DisplayLink Manager'
    killall -q -SIGTERM DisplayLinkUserAgent
    killall -q -SIGTERM DisplayLinkManager
  fi

  wait_for_dl_processes_finished 5
  if are_dl_processes_running; then
    echo "Kill remaining DisplayLink processes"
    killall -q -SIGKILL 'DisplayLink Manager'
    killall -q -SIGKILL DisplayLinkUserAgent
    killall -q -SIGKILL DisplayLinkManager
  fi
}

wait_for_dl_processes_finished()
{
  local timeout=$1

  while [[ $(( timeout-- )) -gt 0 ]]
  do
    if ! are_dl_processes_running; then
      echo "All DisplayLink processes are finished"
      break
    fi

    echo "Wait for dl prcesses to finish"
    sleep 1
  done
}

if [[ $EUID -ne 0 ]]; then
  exec sudo "$0" "$@"
fi

echo ""
echo "Reset Display Preferences v1.2.1"
echo ""
interactive=1
while getopts n opt; do
   case $opt in
     n ) interactive=0;;
     * ) echo "Unknown option"; break;;
  esac
done
errors_occured=false
file_list=$(mktemp)
trap 'rm $file_list' EXIT

terminate_any_remaining_dl_processes
killall cfprefsd

find "/Library/Preferences" -name "com.apple.windowserver.plist*" 2>/dev/null > "$file_list"
find "/Library/Preferences" -name "com.apple.windowserver.displays.plist*" 2>/dev/null >> "$file_list"

for i in ~root /Users/*; do
    find "$i/Library/Preferences/ByHost" -name "com.apple.windowserver.*.plist*" 2>/dev/null >> "$file_list"
    find "$i/Library/Group Containers/73YQY62QM3.com.displaylink.DisplayLinkShared/Library/Preferences" -name "73YQY62QM3.com.displaylink.DisplayLinkShared.plist" 2>/dev/null >> "$file_list"
done

if [[ $(wc -l < "$file_list") -eq 0 ]]; then
    echo "No display preferences found, terminating script"
    [[ "$0" = "${BASH_SOURCE[0]}" ]] && exit 0
fi

echo "This script will delete following files:"
cat "$file_list"

if [[ "${interactive}" -eq 1 ]] ; then
read -p "Do you want to proceed? [Y/N] " -n 1 -r
echo ""
if [[ ! $REPLY =~ ^[Yy]$ ]]; then
    echo "Display preferences has not been reset, terminating script"
    [[ "$0" = "${BASH_SOURCE[0]}" ]] && exit 1
fi
fi
echo "Deleting files..."
while read -r file; do
    echo -n "[delete] : $file"
    if rm -f "$file" 2>/dev/null
    then
        echo " - done"
    else
        errors_occured=true
        echo " - ERROR"
    fi
done < "$file_list"

if [ "$errors_occured" = false ]; then
    echo "All done! Please restart the system"
    exit 0
else
    echo "Errors occurred, please contact customer support"
    exit 1
fi
